<?php

namespace SupNext\Controllers\QuoteSheetGrid;

use Teknisa\Libs\Util\Environment;
use SupNext\Controllers\QuoteSheetGrid\AddRegionController as AddRegion;
use SupNext\Services\QuoteSheetGrid\AddProductService as Service;
use SupNext\Util\Util;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;

class AddProductController {
  private $environment;
  private $addRegion;
  private $service;
  private $util;
  private $error;

  public function __construct(Environment $environment, AddRegion $addRegion, Service $service, Util $util) {
    $this->environment = $environment;
    $this->addRegion = $addRegion;
    $this->service   = $service;
    $this->util      = $util;
  }

  public function addProduct(Request $request, Response $response) {
    try{
      $data = $request->getParameters();
      $data["NRORG"] = $this->environment->getNrorg();
      $data["CDOPERADOR"] = $this->environment->getCdoperador();

      $productExist = $this->verifyProductExist($data);

      if(!empty($productExist)) return $response->setError(new Error("sup::PRODUCT_REGISTERED", 400));

      $this->validateProduct($data, $response);

      $params = $this->prepareParamsProduct($data);

      if ($data["IDUTINDICECOL"] == "S") {
        $params["VRPRECUNIITM"] = $this->calculateVrIndice($data);
        $this->service->insertItemColeWithIndex($params);
      } else {
        $this->service->insertItemColeWithoutIndex($params);
      }

      $this->addRegion->executeProcedureItemcol($data);
      $this->addRegion->processInsertFormPgtoData($data);

      $this->executeProcPrices($data);

      $this->service->updateQuatationParameters($data);

      return $response->addDataSet(new DataSet("addProduct", ["message" => "Inclusão do produto realizado com sucesso.", "status_code" => 200]));
    } catch (\Exception $exception) {
      $response->setError(new Error($exception->getMessage(), 500));
    }
  }

  private function verifyProductExist(array $data): array {
    $conditionsArray = [
      "CDFILIAL"    => $data["CDFILIAL"],
      "NRCOLEPRECO" => $data["NRCOLEPRECO"],
      "CDFORNECED"  => $data["CDFORNECED"],
      "NRORG"       => $data["NRORG"],
      "CDPRODUTO"   => $data["CDPRODUTO"],
    ];

    $productExist = $this->util->filteredDataSet($conditionsArray, "verifyProductExist");

    return $productExist;
  }

  public function validateProduct(array $data, Response $response) {
    $this->validateQualifyBrand($data, $response);
    $this->validateElegibleSupplier($data, $response);
    $this->validateFuncFili($data, $response);
    $this->validateProductSupplier($data, $response);
    $this->validateBrandProduct($data, $response);
    $this->validateEAN($data, $response);
  }

  private function validateQualifyBrand(array $data, Response $response) {
    $conditionsArray = [
      "CDPRODUTO" => $data["CDPRODUTO"],
      "CDMARCA"   => $data["CDMARCA"],
      "CDFILIAL"  => $data["CDFILIAL"],
      "NRORG"     => $data["NRORG"],
      "DTCOL"     => $data["DTCOLEPREC"],
      "DTCOL"     => $data["DTCOLEPREC"],
      "CDFILIAL"  => $data["CDFILIAL"],
      "NRORG"     => $data["NRORG"],
      "CDPRODUTO" => $data["CDPRODUTO"],
      "CDMARCA"   => $data["CDMARCA"],
      "NRORG"     => $data["NRORG"]
    ];

    $qualifyBrand = $this->util->filteredDataSet($conditionsArray, "getQualifyBrand");

    if (!empty($qualifyBrand)) {
      throw new \Exception("sup::BRAND_NOT_QUALIFIED_FOR_PRODUCT", 400);
    }
  }

  private function validateElegibleSupplier(array $data, Response $response) {
    $conditionsArray = [
      "CDPRODUTO"  => $data["CDPRODUTO"],
      "CDFORNECED" => $data["CDFORNECED"],
      "NRORG"      => $data["NRORG"],
      "CDFILIAL"   => $data["CDFILIAL"],
      "NRORG"      => $data["NRORG"],
      "DTCOL"      => $data["DTCOLEPREC"],
      "DTCOL"      => $data["DTCOLEPREC"]
    ];

    $elegibleProduct = $this->util->filteredDataSet($conditionsArray, "getEligibleSupplier");

    if (empty($elegibleProduct)) {
      throw new \Exception("sup::PRODUCT_NOT_QUALIFIED_FOR_SUPPLIERS", 400);
    }
  }

  private function validateFuncFili(array $data, Response $response) {
    $conditionsArray = [
      "CDFILIPROC" => $data["CDFILIPROC"],
      "NRORG"      => $data["NRORG"]
    ];

    $funcFili = $this->util->filteredDataSet($conditionsArray, "getFuncFili");

    if ($funcFili && $funcFili[0]['IDFUNCAOFILI'] == 'R') {
      $conditionsArray = [
        "CDFILICTRL" => $data["CDFILIPROC"],
        "CDREGIAO"   => $data["CDREGIAO"],
        "CDPRODUTO"  => $data["CDPRODUTO"],
        "NRORG"      => $data["NRORG"]
      ];

      $filiReg = $this->util->filteredDataSet($conditionsArray, "getFiliReg");

      if (empty($filiReg)) {
        throw new \Exception("sup::PRODUCT_NOT_NEGOTIATED_BY_REGIONAL_UNIT", 400);
      }

    } else if ($funcFili && $funcFili[0]['IDFUNCAOFILI'] == 'C') {
      $conditionsArray = [
        "CDFILICTRL" => $data["CDFILIPROC"],
        "CDREGIAO"   => $data["CDREGIAO"],
        "CDPRODUTO"  => $data["CDPRODUTO"],
        "NRORG"      => $data["NRORG"]
      ];

      $filiCor = $this->util->filteredDataSet($conditionsArray, "getFiliCor");

      if (empty($filiCor)) {
        throw new \Exception("sup::PRODUCT_NOT_NEGOTIATED_BY_REGIONAL_UNIT", 400);
      }
    }
  }

  private function validateProductSupplier(array $data, Response $response) {
    $conditionsArray = [
      "CDFORNECED" => $data["CDFORNECED"],
      "CDPRODUTO"  => $data["CDPRODUTO"],
      "NRORG"      => $data["NRORG"]
    ];

    $prodFornComp = $this->util->filteredDataSet($conditionsArray, "validateProductSupplier");

    if (empty($prodFornComp)) {
      throw new \Exception("sup::PRODUCT_NOT_REGISTERED_FOR_SUPPLIER", 400);
    }
  }

  private function validateBrandProduct(array $data, Response $response) {
    $conditionsArray = [
      "CDPRODUTO" => $data["CDPRODUTO"],
      "CDMARCA"   => $data["CDMARCA"],
      "NRORG"     => $data["NRORG"]
    ];

    $marcProd = $this->util->filteredDataSet($conditionsArray, "validateBrandProduct");

    if (empty($marcProd)) {
      throw new \Exception("sup::BRAND_NOT_REGISTERD", 400);
    }
  }

  private function validateEAN(array $data, Response $response) {
    $conditionsArray = [
      "CDFORNECED" => $data["CDFORNECED"],
      "NRORG"      => $data["NRORG"]
    ];

    $obrigaEAN = $this->util->filteredDataSet($conditionsArray, "getCodEAN");

    if ($obrigaEAN && $obrigaEAN[0]['IDOBRIGAEAN'] == 'S') {
      $conditionsArray = [
        "CDPRODUTO" => $data["CDPRODUTO"],
        "NRORG"     => $data["NRORG"]
      ];

      $existEAN = $this->util->filteredDataSet($conditionsArray, "verifyExistEAN");

      if (empty($existEAN)) {
        throw new \Exception("sup::EAN_CODE_NOT_REGISTERED_FOR_PRODUCT_BRAND", 400);
      }
    }
  }

  private function prepareParamsProduct(array $data): array {
    $conditionsArray = [
      "CDFILIAL"    => $data["CDFILIAL"],
      "NRCOLEPRECO" => $data["NRCOLEPRECO"],
      "NRORG"       => $data["NRORG"]
    ];

    $priceList = $this->util->filteredDataSet($conditionsArray, "getPriceList")[0];

    $params = [
      "CDFILIAL"        => $data["CDFILIAL"],
      "NRCOLEPRECO"     => $data["NRCOLEPRECO"],
      "CDPRODUTO"       => $data["CDPRODUTO"],
      "CDMARCA"         => $data["CDMARCA"],
      "CDFORNECED"      => $data["CDFORNECED"],
      "CDREGIAO"        => $data["CDREGIAO"],
      "VRPRECUNIITM"    => !empty($data["VRPRECUNIITM"]) ? (float )$data["VRPRECUNIITM"] : 0,
      "CDOPERINCL"      => $data["CDOPERADOR"],
      "IDATUREG"        => $data["IDATUREG"],
      "IDATUFORN"       => $data["IDATUFORN"],
      "NRSEQPRELIST"    => $priceList["NRSEQPRELIST"],
      "IDITVALDIFLISTA" => $priceList["IDITVALDIFLISTA"] ? $priceList["IDITVALDIFLISTA"] : "N",
      "IDPRECALT"       => $priceList["IDPRECALT"] ? $priceList["IDPRECALT"] : "N",
      "VRINDICEITMCOL"  => $data["VRPRECUNIITM"],
      "NRORG"           => $data["NRORG"]
    ];

    return $params;
  }

  public function calculateVrIndice(array $data): string {
    try {
      $conditionsArray = [
        "CDTPINDICE"    => $data["CDTPINDICE"],
        "IDPERVALORCOL" => $data["IDPERVALORCOL"],
        "CDINDICECOL"   => $data["CDINDICECOL"],
        "DTREFINDCOL"   => $data["DTREFINDCOL"],
        "NRORG"         => $data["NRORG"]
      ];

      $vrIndice = $this->util->filteredDataSet($conditionsArray, "getVrIndice");

      if ($vrIndice) {
        $data["VRPRECUNIITM"] = $data["IDPERVALORCOL"] == "2" ?
          $data["VRINDICEITMCOL"] * $vrIndice[0]["VRINDICE"] :
          $data["VRINDICEITMCOL"] * ($vrIndice[0]["VRINDICE"] / 100);
      }

      return $data["VRPRECUNIITM"];
    } catch (Exception $exception) {
      throw new \Exception("Erro ao calcular vr indice: " . $exception->getMessage(), 500);
    }
  }

  public function executeProcPrices(array $data): void {
    if ($data["IDATUREG"]) {
      $data["P_IDATUREG"] = "T";
      $data["P_IDATUFORN"] = $data["IDATUFORN"] ? "T" : "S";
      $data["NEW_CDREGIAO"] = null;

      $this->service->procPrecosToRegiao($data);
    }

    if ($data["IDATUFORN"]) {
      $data["P_IDATUFORN"] = "T";
      $data["P_IDATUREG"] = $data["IDATUREG"] ? "T" : "S";

      $this->service->procPrecosToForn($data);
    }
  }
}
