<?php

namespace SupNext\Controllers\QuoteSheetGrid;

use Teknisa\Libs\Util\Environment;
use SupNext\Util\Util;
use SupNext\Services\QuoteSheetGrid\AddQuotationService as Service;
use Teknisa\Libs\Service\CRUD as CrudService;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;

class AddQuotationController {
  private $environment;
  private $util;
  private $service;
  private $crudService;

  public function __construct(
    Environment $environment,
    Util $util,
    Service $service,
    CrudService $crudService
  ) {
    $this->environment = $environment;
    $this->util        = $util;
    $this->service     = $service;
    $this->crudService = $crudService;
  }

  public function addQuotation(Request $request, Response $response) {
    try {
      $data = $this->processReceivedData($request->getParameters());
      $data["NRORG"] = $this->environment->getNrorg();

      $this->isBuyerRegistered($data, $response);

      $data["CDFILIAL"]   = $this->environment->getCdFilial();
      $data["CDOPERINCL"] = $this->environment->getCdoperador();
      $data["NRCOLEPREC"] = $this->util->generateCode(
        "COLETA" . $data["CDFILIAL"]
      );

      $supplyParameters = $this->getSupplyParameters($data);

      $data["IDSITUCOLETA"] =
        $supplyParameters[0]["IDUTISTATCOT"] === "S" ? "A" : "";

      if ($supplyParameters[0]["IDUTINDICECOL"] === "N") {
        $data["CDTPINDICE"] = null;
        $data["IDPERVALOR"] = null;
        $data["CDINDICE"]   = null;
        $data["DTREFIND"]   = null;
      }

      $this->service->saveQuotation($data);
      $rowsAffect = $this->getRowsAffect($data);

      if ($data["IDCARGOBS"] == "S") $this->service->procedureCargDescSolCot($data);

      if ((!empty($rowsAffect) && $rowsAffect["P_ROWSAFFECT"] === "N")) $this->validateQuoteInclusion($data, $supplyParameters, $response);

      return $response->addDataSet(new DataSet("addQuotation", ["message" => "Inclusão da cotação realizada com sucesso.", "status_code" => 200]));
    } catch (\Exception $exception) {
      throw new \Exception($exception->getMessage(), 500);
    }
  }

  private function processReceivedData(array $data): array {
    $data = $this->util->implodeFields($data, ["CDREGIAO", "CDFORNECED", "CDMARCA", "CDVERIF"]);
    $data["CDPRODFIN"] =
      is_null($data["CDPRODFIN"]) ? $data["CDPRODFIN"] : $this->util->addCaracterInRight($data["CDPRODFIN"]);
    $data["TPRODUTO"] = isset($data["CDPRODINI"]) || isset($data["CDPRODFIN"]) ? null : "T";
    $data["DSOBSCOTPREC"] = empty($data["DSOBSCOTPREC"]) ? "" : $data["DSOBSCOTPREC"];
    $data["IDCARGULTC"] = $data["IDCARGULTC"] === "N" ? "" : $data["IDCARGULTC"];

    return $data;
  }

  private function isBuyerRegistered(array $data, Response $response) {
    try {
      $conditionsArray = [
        "CDOPERADOR" => $data["CDOPERADOR"],
        "NRORG"      => $data["NRORG"],
      ];

      $isRegistered = $this->util->filteredDataSet($conditionsArray, "validateRegistrationBuyer");

      if (empty($isRegistered)) return $response->setError(new Error("sup::BUYER_NOT_REGISTERED", 401));
    } catch (\Exception $exception) {
      throw new \Exception("Erro ao validar comprador registrado.", 500);
    }
  }

  private function getSupplyParameters(array $data): array {
    try {
      $supplyParameters = ["CDFILIAL", "NRORG"];
      $supplyConditions = $this->util->prepareConditions(
        $supplyParameters
      );

      return $this->crudService
        ->getFilteredDataSet("supParams", $supplyConditions)
        ->getRows();
    } catch (\Exception $exception) {
      throw new \Exception("Erro ao pegar parâmetros suprimentos.", 500);
    }
  }

  private function getRowsAffect(array $data): array {
    try {
      if (!$data["NRCOMPRA"] || empty($data["NRCOMPRA"])) {
        $rowsAffect =
          $data["IDTIPOCOTACAO"] == "A"
          ? $this->service->procedureIncludesItemcolTodos($data)
          : $this->service->procedureIncludesItemcolAComprar($data);
      } else {
        $rowsAffect = $this->service->procedureIncludesItemcolProdCompra(
          $data
        );
      }

      return $rowsAffect;
    } catch (\Exception $exception) {
      throw new \Exception("Erro ao pegar linhas afetadas: " . $exception->getMessage(), 500);
    }
  }

  private function validateQuoteInclusion(array $data, array $supParams, Response $response): void {
    if (empty($supParams)) return;

    try {
      $this->validateSupplyParams($data, $supParams, $response);
      $this->validateQuotationType($data, $response);
      $this->validateSupplierProductBuyer($data, $response);
      $this->validateProductBuyer($data, $response);
      $this->validateBrand($data, $supParams, $response);
      $this->validateBrandProductQualification($data, $response);
      $this->validateProductSupplierQualification($data, $response);
      $this->validateRegionForSupplier($data, $response);
      $this->validateFuncBranch($data, $response);
      $this->validateProductInactive($data, $response);
      $this->validateDtQualifySupplier($data, $response);
    } catch (\Exception $exception) {
      throw new \Exception($exception->getMessage(), 500);
    }
  }

  private function validateSupplyParams(array $data, array $supParams, Response $response): void {
    if ($supParams[0]["IDUTLPROCMP"] == "S") {
      $conditionsArray = [
        "CDFILIAL"  => $data["CDFILIAL"],
        "CDPRODINI" => $data["CDPRODINI"],
        "CDPRODFIN" => $data["CDPRODFIN"],
        "TPRODUTO"  => $data["TPRODUTO"],
        "NRORG"     => $data["NRORG"]
      ];

      $prodCmp = $this->util->filteredDataSet($conditionsArray, "validateProdCmp");

      if (empty($prodCmp)) {
        $response->setError(new Error("sup::NO_PURCHASE_PRODUCT", 400));
      }
    }
  }

  private function validateQuotationType(array $data, Response $response): void {
    if ($data["IDTIPOCOTACAO"] == "C") {
      $conditionsArray = [
        "CDPRODINI"   => $data["CDPRODINI"],
        "CDPRODFIN"   => $data["CDPRODFIN"],
        "TPRODUTO"    => $data["TPRODUTO"],
        "DTINIVALCOL" => $data["DTINIVALCOL"],
        "DTFINVALCOL" => $data["DTFINVALCOL"],
        "CDFILIAL"    => $data["CDFILIAL"],
        "CDFILIAL"    => $data["CDFILIAL"],
        "CDREGIAO"    => $data["CDREGIAO"] ?? "T",
        "CDREGIAO"    => $data["CDREGIAO"] ?? "T",
        "CDFILIAL"    => $data["CDFILIAL"],
        "NRORG"       => $data["NRORG"]
      ];

      $request = $this->util->filteredDataSet($conditionsArray, "getRequest");

      if (empty($request) || $request[0]["QTACOMPRAR"] == 0) {
        $response->setError(new Error("sup::NO_PURCHASE_REQUEST", 400));
      }
    }
  }

  private function validateSupplierProductBuyer(array $data, Response $response): void {
    $conditionsArray = [
      "CDFORNECED" => $data["CDFORNECED"] ?? "T",
      "CDFORNECED" => $data["CDFORNECED"] ?? "T",
      "CDPRODINI"  => $data["CDPRODINI"],
      "CDPRODFIN"  => $data["CDPRODFIN"],
      "TPRODUTO"   => $data["TPRODUTO"],
      "CDFILIAL"   => $data["CDFILIAL"],
      "NRORG"      => $data["NRORG"]
    ];

    $request = $this->util->filteredDataSet($conditionsArray, "getFornProdCmp");

    if (empty($request)) {
      $response->setError(new Error("sup::PRODUCT_NOT_REGISTERED_FOR_BUYER_SUPPLIER", 400));
    }
  }

  private function validateProductBuyer(array $data, Response $response): void {
    $conditionsArray = [
      "CDOPERINCL" => $data["CDOPERINCL"],
      "CDPRODINI"  => $data["CDPRODINI"],
      "CDPRODFIN"  => $data["CDPRODFIN"],
      "TPRODUTO"   => $data["TPRODUTO"],
      "CDFILIAL"   => $data["CDFILIAL"],
      "NRORG"      => $data["NRORG"]
    ];

    $prodComp = $this->util->filteredDataSet($conditionsArray, "getProdComp");

    if (empty($prodComp)) {
      $response->setError(new Error("sup::PRODUCT_NOT_REGISTERED_FOR_BUYER", 400));
    }
  }

  private function validateBrand(array $data, array $supParams, Response $response): void {
    if (empty($supParams[0]["CDMARCA"])) {
      $conditionsArray = [
        "CDMARCA"   => $data["CDMARCA"] ?? "T",
        "CDMARCA"   => $data["CDMARCA"] ?? "T",
        "CDPRODINI" => $data["CDPRODINI"],
        "CDPRODFIN" => $data["CDPRODFIN"],
        "TPRODUTO"  => $data["TPRODUTO"],
        "NRORG"     => $data["NRORG"]
      ];

      $marcProd = $this->util->filteredDataSet($conditionsArray, "getBrandProd");

      if (empty($marcProd)) {
        $response->setError(new Error("sup::BRAND_NOT_REGISTERED_OR_QUALIFIED_FOR_PRODUCT", 400));
      }
    }
  }

  private function validateBrandProductQualification(array $data, Response $response): void {
    $conditionsArray = [
      "CDMARCA"   => $data["CDMARCA"] ?? "T",
      "CDMARCA"   => $data["CDMARCA"] ?? "T",
      "CDPRODINI" => $data["CDPRODINI"],
      "CDPRODFIN" => $data["CDPRODFIN"],
      "TPRODUTO"  => $data["TPRODUTO"],
      "CDFILIAL"  => $data["CDFILIAL"],
      "DTCOL"     => $data["DTCOLEPREC"],
      "DTCOL"     => $data["DTCOLEPREC"],
      "CDPRODINI" => $data["CDPRODINI"],
      "CDPRODFIN" => $data["CDPRODFIN"],
      "CDFILIAL"  => $data["CDFILIAL"],
      "NRORG"     => $data["NRORG"]
    ];

    $qualiMarcProd = $this->util->filteredDataSet($conditionsArray, "validateQualifyBrandProd");

    if (empty($qualiMarcProd)) {
      $response->setError(new Error("sup::BRAND_NOT_QUALIFIED_FOR_PRODUCT", 400));
    }
  }

  private function validateProductSupplierQualification(array $data, Response $response): void {
     $conditionsArray = [
      "CDPRODINI"  => $data["CDPRODINI"],
      "CDPRODFIN"  => $data["CDPRODFIN"],
      "TPRODUTO"   => $data["TPRODUTO"],
      "CDFORNECED" => $data["CDFORNECED"] ?? "T",
      "CDFORNECED" => $data["CDFORNECED"] ?? "T",
      "CDFILIAL"   => $data["CDFILIAL"],
      "DTCOL"      => $data["DTCOLEPREC"],
      "DTCOL"      => $data["DTCOLEPREC"],
      "NRORG"      => $data["NRORG"],
      "NRORG"      => $data["NRORG"]
    ];

    $qfoProdForn = $this->util->filteredDataSet($conditionsArray, "validateQfoProdForn");

    if (empty($qfoProdForn)) {
      $response->setError(new Error("sup::PRODUCT_NOT_QUALIFIED_FOR_SUPPLIERS", 400));
    }
  }

  private function validateRegionForSupplier(array $data, Response $response): void {
     $conditionsArray = [
      "CDFORNECED" => $data["CDFORNECED"] ?? "T",
      "CDFORNECED" => $data["CDFORNECED"] ?? "T",
      "CDREGIAO"   => $data["CDREGIAO"] ?? "T",
      "CDREGIAO"   => $data["CDREGIAO"] ?? "T",
      "NRORG"      => $data["NRORG"]
    ];

    $reFoReg = $this->util->filteredDataSet($conditionsArray, "getRegFoReg");

    if (empty($reFoReg)) {
      $response->setError(new Error("sup::SUPPLIERS_NOT_SERVING_REGIONS", 400));
    }
  }

  private function validateFuncBranch(array $data, Response $response): void {
    $conditionsArray = [
      "CDFILIPROC" => $data["CDFILIAL"],
      "NRORG"      => $data["NRORG"]
    ];

    $funcFili = $this->util->filteredDataSet($conditionsArray, "validateFuncBranch");

    if (empty($funcFili)) return;

    if ($funcFili[0]["IDFUNCAOFILI"] == "C") {
      $conditionsArray = [
        "CDFILIPROC" => $data["CDFILIAL"],
        "CDPRODINI"  => $data["CDPRODINI"],
        "CDPRODFIN"  => $data["CDPRODFIN"],
        "TPRODUTO"   => $data["TPRODUTO"],
        "CDREGIAO"   => $data["CDREGIAO"] ?? "T",
        "NRORG"      => $data["NRORG"]
      ];

      $filiCor = $this->util->filteredDataSet($conditionsArray, "validateFiliCor");

      if (empty($filiCor)) {
        $response->setError(new Error("sup::PRODUCT_NOT_NEGOTIATED_BY_CORPORATE_UNIT", 400));
      }
    } else if ($funcFili[0]["IDFUNCAOFILI"] == "R") {
      $conditionsArray = [
        "CDFILIPROC" => $data["CDFILIAL"],
        "CDPRODINI"  => $data["CDPRODINI"],
        "CDPRODFIN"  => $data["CDPRODFIN"],
        "TPRODUTO"   => $data["TPRODUTO"],
        "CDREGIAO"   => $data["CDREGIAO"] ?? "T",
        "NRORG"      => $data["NRORG"]
      ];

      $filiReg = $this->util->filteredDataSet($conditionsArray, "validateFiliReg");

      if (empty($filiReg)) {
        $response->setError(new Error("sup::PRODUCT_NOT_NEGOTIATED_BY_REGIONAL_UNIT", 400));
      }
    }
  }

  private function validateProductInactive(array $data, Response $response): void {
    $conditionsArray = [
      "CDPRODINI" => $data["CDPRODINI"],
      "CDPRODFIN" => $data["CDPRODFIN"],
      "TPRODUTO"  => $data["TPRODUTO"],
      "NRORG"     => $data["NRORG"]
    ];

    $prodInativo = $this->util->filteredDataSet($conditionsArray, "getProdInactive");

    if (!empty($prodInativo)) {
      $response->setError(new Error("sup::PRODUCT_INACTIVE", 400));
    }
  }

  private function validateDtQualifySupplier(array $data, Response $response): void {
    $conditionsArray = [
      "CDFORNECED" => $data["CDFORNECED"] ?? "T",
      "CDFORNECED" => $data["CDFORNECED"] ?? "T",
      "CDFILIAL"   => $data["CDFILIAL"],
      "DTFIN"      => $data["DTFINVALCOL"],
      "DTFIN"      => $data["DTFINVALCOL"],
      "NRORG"      => $data["NRORG"]
    ];

    $dtQualifForn = $this->util->filteredDataSet($conditionsArray, "getDtQualifForn");

    if (!empty($dtQualifForn)) {
      $response->setError(new Error("sup::SUPPLIER_EXPIRED", 400));
    }
  }
}
