import { TekGrid } from '@zeedhi/teknisa-components-common';
import {
  ModalService, Form, LoadingService,
} from '@zeedhi/common';
import { Metadata, Http } from '@zeedhi/core';
import { AppState } from '@/controllers/AppState';
import { Utilities } from '@/util/Utilities';

export class AddProductController {
  private utilities!: Utilities;

  private appState: AppState;

  constructor() {
    this.utilities = new Utilities();
    this.appState = AppState.getInstance();
  }

  public async createModal(): Promise<void> {
    await ModalService.createFromJson(
      '/modals/quoteSheetGrid/addProductModal',
      true,
    );
  }

  public destroyModal(): void {
    ModalService.getModal('addProductModal')?.destroy();
  }

  public openModal(): void {
    const grid = Metadata.getInstance<TekGrid>('multipleLevelsSupplierGrid');
    const row = grid.datasource.currentRow;

    if (row.NRCOLEPRECO) {
      ModalService.getModal('addProductModal').show();
    } else {
      this.utilities.showWarningDialog('Selecione um fornecedor para incluir uma novo produto.');
    }
  }

  public closeModal(): void {
    ModalService.getModal('addProductModal').hide();
  }

  public async includeProduct(): Promise<void> {
    const form = Metadata.getInstance<Form>('addProductForm');
    const quotation = Metadata.getInstance<TekGrid>('multipleLevelsQuotationGrid');
    const region = Metadata.getInstance<TekGrid>('multipleLevelsRegionGrid');
    const supplier = Metadata.getInstance<TekGrid>('multipleLevelsSupplierGrid');
    const isValid = form.validate();

    if (!isValid) return;

    const params = this.processParams(form, quotation, region, supplier);

    LoadingService.show();

    try {
      const { data } = await Http.post('/addProduct', params);

      if (data.error) {
        LoadingService.hide();
        this.utilities.showErrorDialog(data.error);

        return;
      }

      const product = Metadata.getInstance<TekGrid>('multipleLevelsProductsGrid');

      LoadingService.hide();

      this.utilities.showSuccessDialog(
        data.dataset.addProduct.message,
      );

      this.closeModal();
      form.clearForm();
      product.reload();
    } catch (error) {
      LoadingService.hide();
      this.utilities.showErrorDialog('sup::ERROR');
    }
  }

  private processParams(form: Form, quotation: TekGrid, region: TekGrid, supplier: TekGrid) {
    const registeredParams = JSON.parse(localStorage.getItem('REGISTERED_PARAMS'));

    return {
      CDFILIAL: this.appState.CDFILIAL,
      CDFILIPROC: this.appState.CDFILCENTCMP,
      CDFORNECED: supplier.datasource.currentRow.CDFORNECED,
      CDMARCA: form.value.brand.CDMARCA,
      CDOPERINCL: this.appState.CDOPERADOR,
      CDPRODUTO: form.value.product.CDARVPROD,
      CDREGIAO: region.datasource.currentRow.CDREGIAO,
      DTCOLEPREC: quotation.datasource.currentRow.DTCOLEPREC,
      IDATUFORN: false,
      IDATUREG: false,
      IDUTINDICECOL: registeredParams.IDUTINDICECOL,
      NMMARCA: form.value.brand.NMARCA,
      NMPRODUTO: form.value.product.NMPRODUTO,
      NRCOLEPRECO: quotation.datasource.currentRow.NRCOLEPRECO,
      VRPRECUNIITM: form.value.unitPrice ? form.value.unitPrice : 0,
      CDTPINDICECOL: registeredParams.IDUTINDICECOL === 'S' ? quotation.datasource.currentRow.CDTPINDICE : null,
      IDPERVALORCOL: registeredParams.IDUTINDICECOL === 'S' ? quotation.datasource.currentRow.IDPERVALOR : null,
      CDINDICECOL: registeredParams.IDUTINDICECOL === 'S' ? quotation.datasource.currentRow.CDINDICE : null,
      DTREFINDCOL: registeredParams.IDUTINDICECOL === 'S' ? quotation.datasource.currentRow.DTREFIND : null,
    };
  }
}
