import { TekGrid } from '@zeedhi/teknisa-components-common';
import {
  ModalService, Form, LoadingService,
} from '@zeedhi/common';
import { Metadata, Http } from '@zeedhi/core';
import { AppState } from '@/controllers/AppState';
import { Utilities } from '@/util/Utilities';
import type { IFilterLoadRegionPrices, IAddRegionParams } from '@/controllers/modals/interfaces/QuoteSheetGrid';

export class AddRegionController {
  private utilities!: Utilities;

  private appState: AppState;

  constructor() {
    this.utilities = new Utilities();
    this.appState = AppState.getInstance();
  }

  public async createModal(): Promise<void> {
    await ModalService.createFromJson(
      '/modals/quoteSheetGrid/addRegionModal',
      true,
    );
  }

  public destroyModal(): void {
    ModalService.getModal('addRegionModal')?.destroy();
  }

  public openModal(): void {
    const grid = Metadata.getInstance<TekGrid>('multipleLevelsQuotationGrid');
    const row = grid.datasource.currentRow;

    if (row.NRCOLEPRECO) {
      ModalService.getModal('addRegionModal').show();
    } else {
      this.utilities.showWarningDialog('Selecione uma cotação para incluir uma nova região.');
    }
  }

  public closeModal(): void {
    ModalService.getModal('addRegionModal').hide();
  }

  public get filterLoadRegionPrices(): IFilterLoadRegionPrices {
    const grid = Metadata.getInstance<TekGrid>('multipleLevelsQuotationGrid');
    const row = grid.datasource.currentRow;

    return {
      P_CDFILIAL: this.appState.CDFILIAL,
      P_NRCOLEPRECO: row.NRCOLEPRECO,
      P_NRORG: this.appState.NRORG,
    };
  }

  public async includeRegion(): Promise<void> {
    const form = Metadata.getInstance<Form>('addRegionForm');
    const grid = Metadata.getInstance<TekGrid>('multipleLevelsQuotationGrid');
    const region = Metadata.getInstance<TekGrid>('multipleLevelsRegionGrid');
    const isValid = form.validate();

    if (!isValid) return;

    const params = this.processParams(form, grid);

    LoadingService.show();

    try {
      const { data } = await Http.post('/addRegion', params);
      if (data.error) {
        LoadingService.hide();
        this.utilities.showErrorDialog(data.error);

        return;
      }

      LoadingService.hide();

      this.utilities.showSuccessDialog('sup::SUCCESS');

      this.closeModal();
      form.clearForm();
      region.reload();
    } catch (error) {
      LoadingService.hide();
      this.utilities.showErrorDialog('sup::ERROR');
    }
  }

  private processParams(form: Form, grid: TekGrid): IAddRegionParams {
    const { registeredParams } = this.appState;

    return {
      CDREGIAO: form.value.region,
      IDULTIDTVALQ: registeredParams.IDULTIDTVALQ,
      CDREGIAOPREC: form.value.loadRegionPrices?.CDREGIAO ?? null,
      NRINSJURFORN: form.value.forSupplierRegion?.NRINSJURFORN ?? null,
      NMFANTFORN: form.value.forSupplierRegion?.NMFANTFORN ?? null,
      CDFORNECED: form.value.forSupplierRegion?.CDFORNECED ?? null,
      NRCOLEPRECO: grid.datasource.currentRow.NRCOLEPRECO,
      DTCOLEPREC: grid.datasource.currentRow.DTCOLEPREC,
      IDUTINDICECOL: registeredParams.IDUTINDICECOL,
      CDOPERINCL: this.appState.CDOPERADOR,
      CDFILIPROC: this.appState.CDFILIAL,
      CDFILIAL: grid.datasource.currentRow.CDFILIAL,
      NRORG: this.appState.NRORG,
    };
  }
}
