import { TekGrid } from '@zeedhi/teknisa-components-common';
import {
  ModalService,
  Date as DateInput,
  Select,
  SelectMultiple,
  Checkbox,
  Form,
  LoadingService,
} from '@zeedhi/common';
import {
  Metadata, dayjs, IEventParam, Http,
} from '@zeedhi/core';
import { Utilities } from '@/util/Utilities';
import { AppState } from '@/controllers/AppState';
import type { IReplicateQuotationParams } from '@/controllers/modals/interfaces/QuoteSheetGrid';

export class ReplicateQuotationController {
  private utilities!: Utilities;

  private appState: AppState;

  constructor() {
    this.utilities = new Utilities();
    this.appState = AppState.getInstance();
  }

  public async createModal(): Promise<void> {
    await ModalService.createFromJson(
      '/modals/quoteSheetGrid/replicateQuotationModal',
      true,
    );
  }

  public destroyModal(): void {
    ModalService.getModal('replicateQuotationModal')?.destroy();
  }

  public openModal(): boolean {
    const grid = Metadata.getInstance<TekGrid>('multipleLevelsQuotationGrid');

    if (!grid.selectedRows.length) {
      this.utilities
        .showWarningDialog(
          'Selecione uma cotação para realizar a sua replicação.',
        );

      return false;
    }

    if (grid.selectedRows.length > 1) {
      this.utilities.showWarningDialog(
        'Selecione uma única cotação para realizar a sua replicação.',
      );

      return false;
    }

    ModalService.getModal('replicateQuotationModal').show();
    setTimeout(() => {
      this.setDefaultQuotation();

      const { registeredParams } = this.appState;
      const supplier = Metadata.getInstance<Select>('replicateOLSupplier');

      if (registeredParams.IDUTILCONSSOLIOL === 'S') {
        supplier.removeValidation('required');
        supplier.isVisible = false;
      }
    }, 100);

    return true;
  }

  public closeModal(): void {
    ModalService.getModal('replicateQuotationModal').hide();
  }

  public get filterQuotation() {
    return {
      P_CDFILIAL: this.appState.CDFILIAL,
      P_CDCOMPRADOR: 'T',
      P_DTINIVALCOL: null,
      P_DTFINVALCOL: null,
      P_TDATAS: 'T',
      P_CDFORNECED: 'T',
      P_NRORG: this.appState.NRORG,
    };
  }

  public setRegionFieldFilter(): void {
    this.setFilterRegion();
  }

  public setDefaultQuotation(): void {
    const quotation = Metadata.getInstance<Select>('replicateQuotationInput');
    const grid = Metadata.getInstance<TekGrid>('multipleLevelsQuotationGrid');

    quotation.setValue(grid.selectedRows[0].NRCOLEPRECO);
  }

  public getDefaultCollectionDate(): void {
    const date = Metadata.getInstance<DateInput>('collectionDate');
    date.value = dayjs();
  }

  public changeFieldsStates({ component }: IEventParam<Checkbox>): void {
    const quotation = Metadata.getInstance<Select>('replicateQuotationInput');
    const quoteInterval = Metadata.getInstance<DateInput>(
      'dupicateQuoteInterval',
    );
    const buyer = Metadata.getInstance<SelectMultiple>('replicateBuyer');

    if (component.value === 'S') {
      quotation.label = `${quotation.label} *`;
      quotation.addValidation('required');
      quotation.disabled = false;

      quoteInterval.value = [null, null];
      quoteInterval.disabled = true;
      quoteInterval.label = quoteInterval.label.substring(
        0,
        quoteInterval.label.length - 2,
      );

      buyer.value = null;
      buyer.disabled = true;

      this.setFilterRegion();
    } else {
      quotation.disabled = true;
      quotation.label = quotation.label.substring(
        0,
        quotation.label.length - 2,
      );

      quotation.clearValidations();

      quoteInterval.disabled = false;
      quoteInterval.addValidation('required');
      quoteInterval.label = `${quoteInterval.label} *`;

      buyer.disabled = false;

      this.setFilterRegion(false);
    }
  }

  private setFilterRegion(useCurrentQuotation: boolean = true): void {
    const grid = Metadata.getInstance<TekGrid>('multipleLevelsQuotationGrid');
    const region = Metadata.getInstance<any>('replicateRegion');
    const quotation = Metadata.getInstance<Select>('replicateQuotationInput');

    if (!useCurrentQuotation) {
      region.datasource.route = '/crudN/region';
      region.datasource.setFilter({});
    } else {
      region.datasource.setFilter({
        P_CDFILIAL: grid.selectedRows[0].CDFILIAL,
        P_NRCOLEPRECO: quotation.value ?? null,
        P_NRORG: localStorage.getItem('NRORG'),
      });
    }
  }

  public changeRegionState({ component }: IEventParam<Select>): void {
    const region = Metadata.getInstance<SelectMultiple>('OLRegion');

    if (!component.value) {
      region.value = null;
      region.disabled = true;
    } else {
      region.value = null;
      region.disabled = false;
      this.setFilterOLRegion();
    }
  }

  public setFilterOLRegion() {
    const region = Metadata.getInstance<SelectMultiple>('OLRegion');
    const supplier = Metadata.getInstance<Select>('replicateOLSupplier');

    region.datasource.setFilter({ P_CDFORNOPELOG: supplier.value });
  }

  public async replicateQuotation(): Promise<void> {
    const form = Metadata.getInstance<Form>('replicateQuotationModalForm');
    const isValid = form.validate();

    if (!isValid) return;

    LoadingService.show();

    try {
      const params = this.prepareParams();
      const { data } = await Http.post('/replicateQuotation', params);

      if (data.error) {
        this.utilities.showErrorDialog(data.error);
        LoadingService.hide();

        return;
      }

      LoadingService.hide();
      this.utilities.showSuccessDialog('sup::SUCCESS');

      this.closeModal();
      form.clearForm();
      this.getDefaultCollectionDate();

      const quoteGrid = Metadata.getInstance<TekGrid>('multipleLevelsQuotationGrid');
      quoteGrid.reload();
    } catch (error) {
      LoadingService.hide();
      this.utilities.showErrorDialog('sup::ERROR');
    }
  }

  private prepareParams(): IReplicateQuotationParams {
    const form = Metadata.getInstance<Form>('replicateQuotationModalForm');
    const grid = Metadata.getInstance<TekGrid>('multipleLevelsQuotationGrid');
    const registeredParams = JSON.parse(
      localStorage.getItem('REGISTERED_PARAMS'),
    );
    const IDDUPLICA = form.value.NRCOLEPRECO ? '2' : '1';

    return {
      CONSCOTAN:
        registeredParams.IDCOTCONCLUI === 'S'
        && registeredParams.IDUTISTATCOT === 'S'
          ? 'S'
          : 'N',
      CDFILIAL: this.appState.CDFILCENTCMP,
      CDFILIPROC: this.appState.CDFILIAL,
      CDOPERINCL: this.appState.CDOPERADOR,
      NRCOLEPRECO: !form.value.NRCOLEPRECO
        ? grid.selectedRows[0].NRCOLEPRECO
        : form.value.NRCOLEPRECO,
      IDDUPLICA,
      DTINIVALCOL:
        form.value.dupicateQuoteInterval[0] && IDDUPLICA === '1'
          ? form.value.dupicateQuoteInterval[0]
          : dayjs().format('DD/MM/YYYY'),
      DTFINVALCOL:
        form.value.dupicateQuoteInterval[1] && IDDUPLICA === '1'
          ? form.value.dupicateQuoteInterval[1]
          : dayjs().format('DD/MM/YYYY'),
      DTNOVINIVAL: form.value.newQuoteValidityInterval[0],
      DTNOVFINVAL: form.value.newQuoteValidityInterval[1],
      NRSEMAINIC: form.value.dupicateQuoteInterval
        ? this.utilities.getWeekNumber(form.value.newQuoteValidityInterval[0])
        : null,
      NRSEMAFINAL: form.value.dupicateQuoteInterval
        ? this.utilities.getWeekNumber(form.value.newQuoteValidityInterval[1])
        : null,
      DTCOLEPREC: form.value.collectionDate,
      REAJUSTE: form.value.priceAdjustment ? form.value.priceAdjustment : 0,
      CDOPERADOR: form.value.NRCOLEPRECO
        ? null
        : this.appState.CDOPERADOR,
      TCDOPERADOR: this.appState.CDOPERADOR.length ? null : 'T',
      CDREGIAO: form.value.replicateRegion ? form.value.replicateRegion : null,
      CDREGIAOOL: form.value.OLRegion ? form.value.OLRegion : null,
      DUPLICAOL: 'S',
      NRORG: this.appState.NRORG,
    };
  }
}
