/* eslint-disable no-console */
import { TekGrid } from '@zeedhi/teknisa-components-common';
import { DialogService, LoadingService } from '@zeedhi/common';
import {
  Metadata, Http, I18n, IEventParam,
} from '@zeedhi/core';
import { AppState } from '@/controllers/AppState';
import { Utilities } from '@/util/Utilities';
import { AppStateController } from '@/controllers/quoteSheetGrid/AppStateController';
import { ModalsController } from '@/controllers/quoteSheetGrid/ModalsController';
import type {
  IFilterQuotationProductsMasterDetail,
  IFilterSupplierProductsMasterDetail,
} from '@/controllers/interfaces/QuoteSheetGrid';

export class QuoteSheetGridController {
  private utilities!: Utilities;

  private appState: AppState;

  private modals: ModalsController;

  constructor() {
    this.utilities = new Utilities();
    this.appState = new AppStateController().getAppState();
    this.modals = new ModalsController();
  }

  public async createModals(): Promise<void> {
    this.modals.createModals();
  }

  public destroyModals(): void {
    this.modals.destroyModals();
  }

  public get filterBuyer(): { P_CDOPERADOR: string } {
    return { P_CDOPERADOR: this.appState.CDOPERADOR };
  }

  public async updateQuotation({ component, event }: any): Promise<void> {
    event.preventDefault();

    const row = component.getEditedRows();
    row.IDDISMONTLIS = row.IDDISMONTLIS === 'Sim' ? 'S' : 'N';

    this.utilities.updateRow(component, row, component.datasource.route);
  }

  public filterQuoteMasterDetail(): void {
    const quotationGrid = Metadata.getInstance<TekGrid>('multipleLevelsQuotationGrid');
    const quoteSheetGrid = Metadata.getInstance<TekGrid>('quoteSheetGrid');

    quotationGrid.datasource.setFilter(quoteSheetGrid.datasource.filter);
  }

  public get filterQuotationProductsMasterDetail(): IFilterQuotationProductsMasterDetail {
    const quoteGrid = Metadata.getInstance<TekGrid>('multipleLevelsQuotationGrid');

    return {
      P_NRCOLEPRECO: quoteGrid.datasource.currentRow?.NRCOLEPRECO ?? null,
      P_CDFILIAL: this.appState.CDFILIAL,
      P_NRORG: this.appState.NRORG,
    };
  }

  public get filterSupplierProductsMasterDetail(): IFilterSupplierProductsMasterDetail {
    const supplierGrid = Metadata.getInstance<TekGrid>('multipleLevelsSupplierGrid');
    const regionGrid = Metadata.getInstance<TekGrid>('multipleLevelsRegionGrid');

    return {
      P_CDFILIAL: this.appState.CDFILIAL,
      P_NRCOLEPRECO: supplierGrid.datasource.currentRow?.NRCOLEPRECO ?? null,
      P_CDREGIAO: regionGrid.datasource.currentRow?.CDREGIAO ?? null,
      P_CDFORNECED: supplierGrid.datasource.currentRow?.CDFORNECED ?? null,
      P_NRORG: this.appState.NRORG,
    };
  }

  public async validateOperator({ component }: IEventParam<TekGrid>): Promise<void> {
    try {
      const parameters = {
        ...component.datasource.currentRow,
        IDUTINDICECOL: this.appState.registeredParams.IDUTINDICECOL,
        CDCOMPRADOR: this.appState.CDOPERADOR,
      };
      const { data } = await Http.get('/validateOperator', { params: parameters });

      if (data.error) {
        this.utilities.showWarningDialog(data.error);

        if (component.editing) component.cancelEditedRows();
      }

      this.changeQuoteGridState(!data.error === false);
      this.changeProductsState(!data.error);
    } catch (error) {
      this.utilities.showErrorDialog('sup::ERROR');
    }
  }

  private changeQuoteGridState(isEnable: boolean): void {
    const grid = Metadata.getInstance<TekGrid>('multipleLevelsQuotationGrid');
    const disabledActions = [
      'updatePaymentMethod',
      'duplicateQuotation',
      'changeValidityQuotation',
      'loadPurchaseQuantity',
      'loadBestPrices',
      'quoteLog',
    ];

    disabledActions.forEach((action) => {
      const index = grid.actions.map((e) => e.name as any).indexOf(action);
      grid.actions[index].disabled = isEnable;
    });
  }

  private changeProductsState(isEnable: boolean): void {
    const productGrid = Metadata.getInstance<TekGrid>('multipleLevelsProductsGrid');
    productGrid.selectable = isEnable;

    productGrid.actions.forEach((action) => {
      if (action.name !== 'excludeItemsWithZeroPrice') action.disabled = !isEnable;
    });

    productGrid.columns.forEach((column) => {
      if (column.name === 'VRPRECUNIITM' || column.name === 'DSJUSTITM') {
        column.editable = isEnable;
      }
    });

    productGrid.datasource.data = [];
  }

  public async updatePaymentMethod(): Promise<void> {
    const grid = Metadata.getInstance<TekGrid>('multipleLevelsQuotationGrid');

    if (!grid.selectedRows.length) {
      this.utilities
        .showWarningDialog(
          'Selecione uma cotação para realizar a atualização da forma de pagamento.',
        );

      return;
    }

    try {
      await Http.post('/savePaymentMethod', grid.selectedRows);
      this.utilities.showSuccessDialog('sup::SUCCESS');
      grid.reload();
    } catch (error) {
      this.utilities.showErrorDialog('sup::ERROR');
    }
  }

  public async deleteRows({ component, event }: any): Promise<void> {
    event.preventDefault();

    if (component.name === 'multipleLevelsSupplierGrid') {
      const region = Metadata.getInstance<TekGrid>('multipleLevelsRegionGrid');

      component.selectedRows.forEach((row) => {
        row.CDREGIAO = region.datasource.currentRow.CDREGIAO;
      });
    }

    this.utilities.deleteSelectedRows(component, component.selectedRows, component.datasource.route);
  }

  public async deleteRegion({ component, event }: any): Promise<void> {
    event.preventDefault();
    LoadingService.show();

    const quoteGrid = Metadata.getInstance<TekGrid>('multipleLevelsQuotationGrid');
    const parameters = component.selectedRows.map((row) => ({
      ...row,
      CDFILIAL: quoteGrid.datasource.currentRow.CDFILIAL,
    }));

    try {
      await Http.delete('/deleteRegion', { params: parameters });

      LoadingService.hide();

      this.utilities.showSuccessDialog('sup::SUCCESS');

      component.selectedRows = [];
      component.reload();
    } catch (error) {
      LoadingService.hide();
      this.utilities.showErrorDialog('sup::ERROR');
    }
  }

  public async deleteSupplier({ component, event }: any): Promise<void> {
    event.preventDefault();
    LoadingService.show();

    const quoteGrid = Metadata.getInstance<TekGrid>('multipleLevelsQuotationGrid');
    const regionGrid = Metadata.getInstance<TekGrid>('multipleLevelsRegionGrid');
    const parameters = component.selectedRows.map((row) => ({
      ...row,
      CDFILIAL: quoteGrid.datasource.currentRow.CDFILIAL,
      CDREGIAO: regionGrid.datasource.currentRow.CDREGIAO,
    }));

    try {
      await Http.delete('/deleteSupplier', { params: parameters });

      LoadingService.hide();

      this.utilities.showSuccessDialog('sup::SUCCESS');

      component.selectedRows = [];
      regionGrid.reload();
      component.reload();
    } catch (error) {
      LoadingService.hide();
      this.utilities.showErrorDialog('sup::ERROR');
    }
  }

  public async updateProducts({ component, event }: any): Promise<void> {
    event.preventDefault();
    const editedRows = component.getEditedRows();
    const { registeredParams } = this.appState;

    if (registeredParams.IDAUDALTCOT === 'S') {
      for (let i = 0; i < editedRows.length; i += 1) {
        if (
          editedRows[i].DSJUSTITM === null
            || editedRows[i].DSJUSTITM === undefined
        ) {
          this.utilities.showWarningDialog(I18n.translate('Justificativa não preenchida.'));

          return;
        }
      }
    }

    this.showDialogs(registeredParams, editedRows);

    const params = this.prepareParamsUpdateProduct(editedRows);

    try {
      const { data } = await Http.post('/updateProducts', params);

      if (data.error) {
        this.utilities.showErrorDialog(data.error);

        return;
      }

      component.cancelEditedRows();
    } catch (error) {
      this.utilities.showErrorDialog('sup::ERROR');
    }
  }

  private showDialogs(registeredParams, editedRows) {
    if (registeredParams.IDPERMREPLICREG === 'S') {
      this.showRegionDialog(registeredParams, editedRows);
    } else {
      editedRows.forEach((row) => {
        row.IDATUREG = false;
      });
    }
  }

  private showRegionDialog(registeredParams, editedRows): void {
    const confirmEvent: any = {
      click: () => {
        editedRows.forEach((row) => {
          row.IDATUREG = true;
        });

        if (registeredParams.IDPERMREPLICFOR === 'S') {
          this.showSupplierDialog(registeredParams, editedRows);
        } else {
          editedRows.forEach((row) => {
            row.IDATUFORN = false;
          });
        }
      },
    };

    const cancelEvent: any = {
      click: () => {
        editedRows.forEach((row) => {
          row.IDATUREG = false;
        });

        if (registeredParams.IDPERMREPLICFOR === 'S') {
          this.showSupplierDialog(registeredParams, editedRows);
        } else {
          editedRows.forEach((row) => {
            row.IDATUFORN = false;
          });
        }
      },
    };

    DialogService.show({
      name: 'buttons-dialog',
      type: 'warning',
      title: I18n.translate('Atenção'),
      text: I18n.translate('Deseja atualizar as alterações para as outras regiões?'),
      buttons: [
        {
          name: 'yes',
          component: 'ZdButton',
          label: I18n.translate('Sim'),
          events: confirmEvent,
        },
        {
          name: 'no',
          component: 'ZdButton',
          label: I18n.translate('Não'),
          events: cancelEvent,
        },
      ],
    });
  }

  private showSupplierDialog(registeredParams, editedRows): void {
    const confirmEvent: any = {
      click: () => {
        editedRows.forEach((row) => {
          row.IDATUFORN = true;
          DialogService.hide();
        });
      },
    };

    const cancelEvent: any = {
      click: () => {
        editedRows.forEach((row) => {
          row.IDATUFORN = false;
        });
        DialogService.hide();
      },
    };

    DialogService.show({
      name: 'buttons-dialog',
      type: 'warning',
      title: I18n.translate('Atenção'),
      text: I18n.translate('Deseja atualizar as alterações para as outros fornecedores?'),
      buttons: [
        {
          name: 'yes',
          component: 'ZdButton',
          label: I18n.translate('Sim'),
          events: confirmEvent,
        },
        {
          name: 'no',
          component: 'ZdButton',
          label: I18n.translate('Não'),
          events: cancelEvent,
        },
      ],
    });
  }

  private prepareParamsUpdateProduct(editedRows) {
    const quoteSheetGrid = Metadata.getInstance<TekGrid>('multipleLevelsQuotationGrid');

    editedRows.forEach((row) => {
      row.DTCOLEPREC = quoteSheetGrid.datasource.currentRow.DTCOLEPREC;
      row.CDFILIPROC = this.appState.CDFILCENTCMP;

      if (!row.DSJUSTITM) row.DSJUSTITM = '';
    });

    return editedRows;
  }

  public async deleteProduct({ component, event }: any): Promise<void> {
    event.preventDefault();
    LoadingService.show();

    try {
      await Http.delete('/deleteProduct', { params: component.selectedRows });
      LoadingService.hide();

      this.utilities.showSuccessDialog('sup::SUCCESS');

      component.selectedRows = [];
      component.reload();
    } catch (error) {
      LoadingService.hide();
      this.utilities.showErrorDialog('sup::ERROR');
    }
  }

  public async excludeItemsWithZeroPrice(): Promise<void> {
    const supplierGrid = Metadata.getInstance<TekGrid>('multipleLevelsSupplierGrid');

    if (!supplierGrid.datasource.currentRow.CDFORNECED) {
      this.utilities.showWarningDialog('Selecione um fornecedor para excluir os itens com preços zerados.');

      return;
    }

    LoadingService.show();

    try {
      const quoteGrid = Metadata.getInstance<TekGrid>('multipleLevelsQuotationGrid');
      const parameters = {
        CDFILIAL: quoteGrid.datasource.currentRow.CDFILIAL,
        NRCOLEPRECO: quoteGrid.datasource.currentRow.NRCOLEPRECO,
        NRORG: this.appState.NRORG,
      };

      const { data } = await Http.delete('/excludeItemsWithZeroPrice', { params: parameters });

      if (data.error) {
        LoadingService.hide();
        this.utilities.showErrorDialog(data.error);

        return;
      }

      LoadingService.hide();
      this.utilities.showSuccessDialog('sup::SUCCESS');
      quoteGrid.reload();
    } catch (error) {
      LoadingService.hide();
      this.utilities.showErrorDialog('sup::ERROR');
    }
  }

  public async loadPurchaseQuantity(): Promise<void> {
    const quoteGrid = Metadata.getInstance<TekGrid>('multipleLevelsQuotationGrid');

    if (!quoteGrid.selectedRows.length) {
      this.utilities.showWarningDialog('Selecione uma cotação para atualizar a quantidade a comprar.');

      return;
    }

    LoadingService.show();

    try {
      const { data } = await Http.put('/loadPurchaseQuantity', { params: quoteGrid.selectedRows });

      if (data.error) {
        LoadingService.hide();
        this.utilities.showErrorDialog(data.error);

        return;
      }

      LoadingService.hide();
      this.utilities.showSuccessDialog('sup::SUCCESS');
    } catch (error) {
      LoadingService.hide();
      this.utilities.showErrorDialog('sup::ERROR');
    }
  }
}
